#!/usr/bin/env python
# encoding: utf-8
from ctypes import *
import os, glob

class LocalMetadata(Structure):
    _fields_ = [
            ("start_timestamp", c_double),
            ("end_timestamp", c_double),
            ("num_files", c_int),
            ("total_records", c_int),
            ("filemap", POINTER(c_char_p)),
            ("file_sizes", POINTER(c_size_t)),
            ("function_count", c_int*256),
    ]

    # In Python3, self.filemap[i] is 'bytes' type
    # For compatable reason, we convert it to str type
    # and will only use self.filenames[i] to access the filename
    def filemap_to_strs(self):
        self.filenames = [''] * self.num_files
        for i in range(self.num_files):
            if type(self.filemap[i]) != str:
                self.filenames[i] = self.filemap[i].decode('utf-8')
            else:
                self.filenames[i] = self.filemap[i]

class GlobalMetadata(Structure):
    _fields_ = [
            ("time_resolution", c_double),
            ("total_ranks", c_int),
            ("compression_mode", c_int),
            ("peephole_window_size", c_int),
    ]


class Record(Structure):
    _fields_ = [
            ("status", c_char),
            ("tstart", c_double),
            ("tend", c_double),
            ("func_id", c_ubyte),
            ("arg_count", c_int),
            ("args", POINTER(c_char_p)),    # Note in python3, args[i] is 'bytes' type
            ("res", c_int),
    ]

    # In Python3, self.args[i] is 'bytes' type
    # For compatable reason, we convert it to str type
    # and will only use self.arg_strs[i] to access the filename
    def args_to_strs(self):
        arg_strs = [''] * self.arg_count
        for i in range(self.arg_count):
            if(type(self.args[i]) == str):
                arg_strs[i] = self.args[i]
            else:
                arg_strs[i] = self.args[i].decode('utf-8')
        return arg_strs

'''
GM: Global Metadata
LMs: List of Local Metadata
records: List (# ranks) of Record*, each entry (Record*) is a list of records for that rank
'''
class RecorderReader:
    def str2char_p(self, s):
        return c_char_p( s.encode('utf-8') )

    def __init__(self, logs_dir):
        current_dir = os.path.abspath(os.path.dirname(__file__))
        parent_dir = os.path.abspath(os.path.join(current_dir, os.pardir))
        lib_dir = os.path.abspath(os.path.join(parent_dir, 'lib'))
        search_path = os.path.abspath(os.path.join(lib_dir, 'librreader*.so'))

        libreader_path = ''
        found = glob.glob(search_path)
        if len(found) == 1:
            libreader_path = found[0]

        libreader = cdll.LoadLibrary(libreader_path)
        libreader.read_records.restype = POINTER(Record)

        self.GM = GlobalMetadata()
        self.LMs = []
        self.records = []

        libreader.read_global_metadata(self.str2char_p(logs_dir + "/recorder.mt"), pointer(self.GM))
        self.load_func_list(logs_dir + "/recorder.mt")

        for rank in range(self.GM.total_ranks):
            print("[recorder-utils] Read trace file for rank: " + str(rank))
            LM = LocalMetadata()
            libreader.read_local_metadata(self.str2char_p(logs_dir+"/"+str(rank)+".mt"), pointer(LM))
            LM.filemap_to_strs()
            self.LMs.append(LM)


            local_records  = libreader.read_records(self.str2char_p(logs_dir+"/"+str(rank)+".itf"), LM.total_records, pointer(self.GM))
            libreader.decompress_records(local_records, LM.total_records)

            self.records.append(local_records)


    def load_func_list(self, global_metadata_path):
        with open(global_metadata_path, 'rb') as f:
            f.seek(24, 0)
            self.funcs = f.read().splitlines()
            self.funcs = [func.decode('utf-8').replace("PMPI", "MPI") for func in self.funcs]

#if __name__ == "__main__":
#    import sys
#    reader = RecorderReader(sys.argv[1])



#!/usr/bin/env python
# encoding: utf-8
import os, bokeh

def relpath(path):
    return os.path.relpath(path)

css_style = """
<style>
  html { font-size: 12px; }
  h2 { font-size: 14px; }
  h4 { font-size: 12px; }
  table {
    font-size: 12px;
    border-collapse: collapse;
  }
  th, td {
    border: 1px solid #ccc;
    padding: 4px;
    text-align: left;
  }
  tr:nth-child(even) {
    background-color: #eee;
  }
  tr:nth-child(odd) {
    background-color: #fff;
  }
  .content {
    max-width: 1000px;
    margin: auto;
  }
</style>
"""


class HTMLWriter:

    def __init__(self, path):
        self.path = path
        self.bokeh_version = bokeh.__version__  # python bokeh version must match the JS version
        # 0
        self.performanceTable = ""
        self.recordCount = ""               # 0.1

        self.fileCount = ""                 # 1.1
        self.fileAccessModeTable = ""       # 1.2

        # 2.
        self.functionLayers = ""            # 2.1
        self.functionPatterns = ""          # 2.2
        self.functionCount = ""             # 2.3
        self.functionTimes = ""             # 2.4

        # 3.
        self.overallIOActivities = ""
        self.offsetVsRank = ""
        self.offsetVsTime = ""
        self.fileAccessPatterns = ""

        # 4.
        self.readIOSizes = ""
        self.writeIOSizes = ""

    def get_html_head(self):
        html_head = """
            <head>
                %s
                <meta charset="UTF-8">
                <script src="https://cdn.bokeh.org/bokeh/release/bokeh-%s.min.js"></script>
                <script src="https://cdn.bokeh.org/bokeh/release/bokeh-widgets-%s.min.js"></script>
                <script src="https://cdn.bokeh.org/bokeh/release/bokeh-tables-%s.min.js"></script>
            </head>
        """ %(css_style, self.bokeh_version, self.bokeh_version, self.bokeh_version)
        return html_head

    def write_html(self):
        html_content  = """
        <html>
            %s
            <body><div class="content">
                <h2> 0. Performance </h2>
                %s
                <h4> 0.1 Record Count </h4>
                %s
                <h2> 1. File Statistics </h2>
                <h4> 1.1 Number of file accessed by each rank</h4>
                %s
                <!--
                <h4> 1.2 Access mode of each file </h4>
                <div style="height:400px; overflow:auto;">
                %s
                </div>
                -->
                <hr>

                <h2> 2. Function Statistics </h3>
                <div>
                    <div style="display:inline-block">
                        <h4> 2.1 I/O Layers</h4>
                        %s
                    </div>
                    <div style="display:inline-block">
                        <h4> 2.2 POSIX I/O Patterns </h4>
                        %s
                    </div>
                </div>
                <h4> 2.3 Function count </h4>
                %s
                <hr>
                <h4> 2.4 Seconds spent on each function </h4>
                %s
                <hr>


                <h2> 3. Access Patterns </h2>
                <h4> 3.1 Overall I/O activities </h4>
                %s
                <h4> 3.2 Accessed offsets VS ranks </h4>
                %s
                <h4> 3.3 Accessed offsets VS time </h4>
                %s
                <h4> 3.4 File access patterns </h4>
                <div style="height:400px; overflow:auto;">
                %s
                </div>
                <hr>

                <h2> 4. Count of I/O access sizes </h2>
                <div>
                    <div style="display:inline-block">
                        <h4> 4.1 Read </h4>
                        %s
                    </div>
                    <div style="display:inline-block">
                        <h4> 4.2 Write </h4>
                        %s
                    </div>
                </div>
            </div></body>
        </html>
        """ %(self.get_html_head(), self.performanceTable, self.recordCount, self.fileCount, self.fileAccessModeTable, \
                self.functionLayers, self.functionPatterns, self.functionCount, self.functionTimes, \
                self.overallIOActivities, self.offsetVsRank, self.offsetVsTime, self.fileAccessPatterns, \
                self.readIOSizes, self.writeIOSizes)

        f = open("./recorder-report.html", "w")
        f.write(html_content)
        f.close()

        #self.write_pdf(html_content)


    def write_pdf(self, html_content):
        from xhtml2pdf import pisa
        f = open("./reports.out/simple_report.pdf", "w+b")
        status = pisa.CreatePDF(
            html_content,                # the HTML to convert
            dest=f)           # file handle to recieve result
        f.close()



import setuptools

c_reader_module = setuptools.Extension('lib/librreader',
                                        ['lib/reader.c'], include_dirs=['lib'])

'''
import subprocess
from setuptools.command.build_py import build_py
class my_build(build_py):
    def run(self):
        #_build.run(self)
        subprocess.call(['make', 'clean', '-C', 'lib'])
        subprocess.call(['make', '-C', 'lib'])
        print("HHH")
'''

with open("README.md", "r") as fh:
    long_description = fh.read()


setuptools.setup(
    name="recorder-viz",                # Package name, e.g., pip install recorder-viz
    version="0.1.3",
    author="Chen Wang",
    author_email="wangvsa@gmail.com",
    description="Utilities for processing Recorder traces",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/wangvsa/recorder-viz",
    packages=['recorder_viz', 'lib'],          # package for import: after installaion, import recorder_viz
    package_data = {'lib': ['*.h']},            # *.h by default will not be copied, we use this to ship it.
    classifiers=[
        "Programming Language :: Python :: 2.7",
        "License :: OSI Approved :: University of Illinois/NCSA Open Source License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=2.7',
    ext_modules=[c_reader_module],
    #cmdclass={'build_py': my_build},
)

